/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: phk_lineup.c,v 1.1 2008/02/15 10:18:37 phk Exp $
 *
 * Functions for assembling a bytestream into text-lines and calling
 * a function on each.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include "phk_lineup.h"
#include "miniobj.h"

struct lineup {
	unsigned	magic;
#define LINEUP_MAGIC	0x8286661
	char		*buf;
	unsigned	bufl;
	unsigned	bufp;
	void		*priv;
	lineup_f	*func;
};

struct lineup *
NewLineup(void *priv, lineup_f *func)
{
	struct lineup *l;

	ALLOC_OBJ(l, LINEUP_MAGIC);
	l->func = func;
	l->priv = priv;
	l->bufl = BUFSIZ;
	l->buf = malloc(l->bufl + 1);
	assert(l->buf != NULL);
	return (l);
}

void
DestroyLineup(struct lineup *l)
{

	CHECK_OBJ_NOTNULL(l, LINEUP_MAGIC);
	free(l->buf);
	FREE_OBJ(l);
}

static int
LineUpProcess(struct lineup *l)
{
	char *p, *q;
	int i;

	l->buf[l->bufp] = '\0';
	for (p = l->buf; *p != '\0'; p = q) {
		q = strchr(p, '\n');
		if (q == NULL)
			break;
		*q++ = '\0';
		i = l->func(l->priv, p);
		if (i != 0)
			return (i);
	}
	if (*p != '\0') {
		q = strchr(p, '\0');
		assert(q != NULL);
		l->bufp = (unsigned)(q - p);
		memmove(l->buf, p, l->bufp);
		l->buf[l->bufp] = '\0';
	} else
		l->bufp = 0;
	return (0);
}

int
LineUpFd(int fd, struct lineup *l)
{
	int i;

	CHECK_OBJ_NOTNULL(l, LINEUP_MAGIC);
	i = read(fd, l->buf + l->bufp, l->bufl - l->bufp);
	if (i <= 0)
		return (-1);
	l->bufp += i;
	return (LineUpProcess(l));
}

int
LineUpFile(FILE *f, struct lineup *l)
{
	char *p;

	CHECK_OBJ_NOTNULL(l, LINEUP_MAGIC);
	p = fgets(l->buf + l->bufp, l->bufl - l->bufp, f);
	if (p == NULL)
		return (-1);
	l->bufp = strlen(l->buf);
	return (LineUpProcess(l));
}
