/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: phk_tcp.c,v 1.5 2006/09/09 19:25:44 phk Exp $
 *
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include "phk_tcp.h"

#define AZ(foo) assert((foo) == 0)

int
PhkTcpOpen(const char *host, const char *port)
{
	char *copy = NULL, *p;
	struct addrinfo *res, *res0, hint;
	int s, i, error;

	memset(&hint, 0, sizeof hint);

	hint.ai_family = PF_UNSPEC;
	hint.ai_socktype = SOCK_STREAM;
	if (port == NULL) {
		copy = strdup(host);
		if (copy == NULL) {
			fprintf(stderr, "Cannot allocate memory\n");
			return (-1);	/* XXX: error message */
		}
		host = copy;
		p = strchr(host, ':');
		if (p == NULL) {
			fprintf(stderr, "No port specified\n");
			return (-1);	/* XXX: error message */
		}
		*p = '\0';
		port = p + 1;
	}
	error = getaddrinfo(host, port, &hint, &res);
	if (error) {
		fprintf(stderr, "getaddrinfo(%s, %s) failed: %s\n",
		    host, port, gai_strerror(error));
		if (copy != NULL)
			free(copy);
		return (-1);	/* XXX: error message */
	}

	res0 = res;
	do {
		s = socket(res0->ai_family, res0->ai_socktype,
		    res0->ai_protocol);
		if (s < 0)
			continue;
		i = 1;
		AZ(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof i));

		error = connect(s, res0->ai_addr, res0->ai_addrlen);
		if (!error) 
			break;
		close(s);
		s = -1;
	} while ((res0 = res0->ai_next) != NULL);

	freeaddrinfo(res);

	if (s >= 0) {
		i = 1;
		AZ(setsockopt(s, SOL_SOCKET, SO_NOSIGPIPE, &i, sizeof i));
	}
	if (copy != NULL)
		free(copy);
	return (s);
}
