/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: test_expr.c,v 1.2 2008/04/29 16:35:38 phk Exp $
 *
 */

#include <stdio.h>
#include <math.h>
#include <err.h>

#include "phk_expr.h"

static void
test(double e, const char *s)
{
	double d;
	char *ers = NULL;

	d = phk_expr(s, &ers);
	if (isnan(d) && isnan(e) && ers != NULL) {
		printf("Expected error:\n%s", ers);
	} else if (ers != NULL) {
		errx(1, "Got error:\n%s", ers);
	} else if (isnan(e) && ers == NULL) {
		errx(1, "Missing error");
	} else if (fabs(e - d) > 1e-10) {
		errx(1, "Failed: \"%s\" gives %g instead of %g (%g)",
		    s, d, e, d - e);
	} else {
		printf("\"%s\" -> %g\n", s, d);
	}
}

#define TEST(str)	test((str), #str)

int
main(int argc __unused, char **argv __unused)
{

	setbuf(stderr, NULL);
	setbuf(stdout, NULL);
	TEST( 3.0 );
	TEST(3 + 5.0);
	TEST(5 - 3.0);
	TEST(6 / 3.0);
	TEST(6 * 3.0);
	TEST(2 + 3 * 4);
	TEST((2 + 3) * 4);
	TEST((2 + (3 / 3)) * (4 - 1) + 3 );
	TEST(2 + 3 - 1 + 2 - 1 + 3);
	test(NAN,	"2 + a");
	test(2.0,	"6 [V] / 3.0 [u/V]");
	TEST(-2);
	TEST(-(2));
	TEST(- 2);
	TEST(- 2);
	TEST(0 - 2);
	TEST(0 - +2);
	TEST(2 - - 2);
	TEST(-2 - - 2);
	TEST(- 2 - - 2);
	TEST(- 2 - +2);
	TEST(- 2 - - - 2);
	test(216.0,	"6 [V] ^ 3.0 [u/V]");
	TEST(sqrt(4));
	TEST(sin(3.14159265));
	TEST(sin (3.14159265));
	test(NAN,	"sin(3.1415");
	test(NAN,	"sin 3.1415)");
	test(NAN,	"klyf(3.1415)");
	TEST(round(4.49));
	TEST(ceil(4.1));
	TEST(floor(4.1));
	test(2, "log2(4)");
	test(14.4,	"2.40 [VPC] * 6");
	test(3104,	"((2.40 [V/cl] * 6 [cl]) - 8.192 [V]) / .002 [V/step]");
	test(2714,	"((2.27 [V/cl] * 6 [cl]) - 8.192 [V]) / .002 [V/step]");
	printf("Test completed, no errors found\n");
	return (0);
}
